@@ -7,9 +7,7 @@ import cv2
 import numpy as np
 from os.path import isabs, realpath, join, dirname
 from scipy import sparse
-import tensorflow as tf
 
-from .CTC_nets import build_eval_net, line_nets
 from .softmax import softmax
 
 
@@ -28,12 +26,23 @@ class BaseEngineLineOCR(object):
 
         self.characters = tuple(self.config['characters'])
         self.net_name = self.config['net_name']
+        if "embed_num" in self.config:
+            self.embed_num = int(self.config["embed_num"])
+        else:
+            self.embed_num = None
+        if "embed_id" in self.config:
+            if self.config["embed_id"] != "mean":
+                self.embed_id = int(self.config["embed_id"])
+            else:
+                self.embed_id = "mean"
+        else:
+            self.embed_id = None
         self.gpu_id = gpu_id
 
         self.batch_size = batch_size
 
         self.line_padding_px = 32
-
+        self.max_input_horizontal_pixels = 480 * batch_size
 
     def process_lines(self, lines, sparse_logits=True, tight_crop_logits=False):
         """Runs ocr network on multiple lines.
@@ -54,23 +63,27 @@ class BaseEngineLineOCR(object):
 
         all_transcriptions = [None]*len(lines)
         all_logits = [None]*len(lines)
+        all_logit_coords = [None]*len(lines)
 
         #  process all lines ordered by their length
-        line_ids = [x for x, y in sorted(enumerate(lines), key=lambda x: x[1].shape[1])]
+        line_ids = [x for x, y in sorted(enumerate(lines), key=lambda x: -x[1].shape[1])]
         while line_ids:
-            batch_line_ids = line_ids[:self.batch_size]
-            line_ids = line_ids[self.batch_size:]
-            max_width = 0
-            for ids in batch_line_ids:
-                max_width = max(max_width, lines[ids].shape[1])
-
+            max_width = lines[line_ids[0]].shape[1]
             max_width = int(np.ceil(max_width / 32.0) * 32)
+            batch_size = max(1, self.max_input_horizontal_pixels // max_width)
+
+            batch_line_ids = line_ids[:batch_size]
+            line_ids = line_ids[batch_size:]
 
             batch_data = np.zeros(
-                [self.batch_size, self.line_px_height, max_width + 2*self.line_padding_px, 3], dtype=np.uint8)
+                [len(batch_line_ids), self.line_px_height, max_width + 2*self.line_padding_px, 3], dtype=np.uint8)
             for data, ids in zip(batch_data, batch_line_ids):
                 data[:, self.line_padding_px:self.line_padding_px+lines[ids].shape[1], :] = lines[ids]
 
+            if batch_data.shape[2] > self.max_input_horizontal_pixels:
+                print(f'WARNING: Line too long for OCR engine. Cropping from {batch_data.shape[2]} px down to {self.max_input_horizontal_pixels}.')
+                batch_data = batch_data[:, :, :self.max_input_horizontal_pixels]
+
             out_transcriptions, out_logits = self.run_ocr(batch_data)
 
             for ids, transcription, line_logits in zip(batch_line_ids, out_transcriptions, out_logits):
@@ -79,23 +92,31 @@ class BaseEngineLineOCR(object):
                     line_logits = line_logits[
                                   int(self.line_padding_px // self.net_subsampling):int(
                                      (self.line_padding_px + lines[ids].shape[1]) // self.net_subsampling)]
-                #else:
+                    all_logit_coords[ids] = [None, None]
+                    #else:
                 #    line_logits = line_logits[
                 #              int(self.line_padding_px // self.net_subsampling - 2):int(
                 #                  lines[ids].shape[1] // self.net_subsampling + 8)]
+                else:
+                    all_logit_coords[ids] = [
+                        int(self.line_padding_px // self.net_subsampling),
+                        int((self.line_padding_px + lines[ids].shape[1]) // self.net_subsampling)]
                 if sparse_logits:
                     line_probs = softmax(line_logits, axis=1)
                     line_logits[line_probs < 0.0001] = 0
                     line_logits = sparse.csc_matrix(line_logits)
                 all_logits[ids] = line_logits
 
-        return all_transcriptions, all_logits
+        return all_transcriptions, all_logits, all_logit_coords
 
 
 class EngineLineOCR(BaseEngineLineOCR):
     def __init__(self, json_def, gpu_id=0, batch_size=8):
         super(EngineLineOCR, self).__init__(json_def, gpu_id=0, batch_size=8)
 
+        import tensorflow as tf
+        from .CTC_nets import build_eval_net, line_nets
+
         self.net_graph = tf.Graph()
         tf.reset_default_graph()
         with self.net_graph.as_default():
@@ -156,7 +177,7 @@ def test_line_ocr(line_list, ocr_engine_json):
             raise ValueError('Error: Could not read image "{}"'.format(line))
         lines.append(line_img)
 
-    transcriptions, logits = ocr_engine.process_lines(lines)
+    transcriptions, logits, _ = ocr_engine.process_lines(lines)
 
     for transcription, line in zip(transcriptions, lines):
         print(transcription)
